﻿#region

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

using NHibernate.BusinessObjects;

using Tutorial;

#endregion


namespace Console
{
	internal class Program
	{
		private static void Main()
		{
			var files = Directory.GetFiles(Path.Combine(AppDomain.CurrentDomain.BaseDirectory,
			                                            @"..\..\..\DataModel"), "*.bo.xml",
			                               SearchOption.AllDirectories);

			var document = BusinessDocumentFactory.CreateDocument(files);

			document.Create(@"localhost\SqlExpress", "Tutorial", null, null);
			//document.Open(@"localhost\SqlExpress", "Tutorial", null, null);

			try
			{
				using (var session = document.OpenSession())
				{
					var person1 = CreatePerson(session);

					FillAddress(person1);

					var persons = GetPersonsByName(document, "Doe");
				}
			}
			finally
			{
				document.Close();
			}
		}


		private static IPerson CreatePerson(IBusinessSession session)
		{
			using (var transaction = session.BeginTransaction())
			{
				var person = session.CreatePerson();
				person.Name = "Doe";
				person.FirstName = "John";
				person.Age = 32;

				transaction.Commit();

				return person;
			}
		}


		private static IList<IPerson> GetPersonsByName(IBusinessDocument document, string name)
		{
			var session = document.CurrentSession;

			return (from person in session.Linq<IPerson>()
			        where person.Name == name
			        select person).ToList();
		}


		private static void FillAddress(IPerson person)
		{
			var address = person.Address;

			using (var transaction = person.Session.BeginTransaction())
			{
				address.City = "New York";
				address.Street = "72 Spring Street";
				address.Zip = "10012";

				transaction.Commit();
			}
		}
	}
}