
using System;
using NHibernate.BusinessObjects;
using Iesi.Collections.Generic;


namespace Tutorial
{
	// Business object implementation

	[Serializable]
	public sealed class Person : BusinessObject, IPerson, IVisitable
	{
	
		private Person()
		{
			try
			{
			}
			catch(Exception e)
			{
				throw new BusinessException(EBusinessError.FatalError, "Could not create object of type 'Tutorial.Person'.", e);
			}
		}
		

		#region IPerson Properties


		private System.String _name;
		public System.String Name 
		{ 
			get
			{
				try
				{
					return _name;
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.ReadError, "Could not read value from property 'Name'.", e);
				}
			} 
			set
			{
				try
				{
					TestSession(); 
					TestObjectLock();
					_name = value;
					SetModified();
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.WriteError, "Could not write value to property 'Name'.", e);
				}
			} 
		}
		

		private System.String _firstName;
		public System.String FirstName 
		{ 
			get
			{
				try
				{
					return _firstName;
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.ReadError, "Could not read value from property 'FirstName'.", e);
				}
			} 
			set
			{
				try
				{
					TestSession(); 
					TestObjectLock();
					_firstName = value;
					SetModified();
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.WriteError, "Could not write value to property 'FirstName'.", e);
				}
			} 
		}
		

		private System.Int32 _age;
		public System.Int32 Age 
		{ 
			get
			{
				try
				{
					return _age;
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.ReadError, "Could not read value from property 'Age'.", e);
				}
			} 
			set
			{
				try
				{
					TestSession(); 
					TestObjectLock();
					_age = value;
					SetModified();
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.WriteError, "Could not write value to property 'Age'.", e);
				}
			} 
		}
		
		private Tutorial.IAddress _address;
		public Tutorial.IAddress Address 
		{ 
			get
			{
				try
				{
					TestSession(_address); 
					return _address;
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.ReadError, "Could not get child object from property 'Address'.", e);
				}
			} 
			private set
			{
				try
				{
					_address = value;
				}
				catch (Exception e)
				{
					throw new BusinessException(EBusinessError.WriteError, "Could not set child object to property 'Address'.", e);
				}
			} 
		}


		#endregion
		
		#region Methods
		
		#endregion //Methods

		#region Acyclic visitor
		
		public override T Apply<T>(IVisitor<T> visitor)
		{
			var specificVisitor = visitor as IPersonVisitor<T>;
			
			return specificVisitor != null ? specificVisitor.Visit(this) : base.Apply(visitor);
		}
		
		#endregion //Acyclic visitor
	}
}


